<script setup>

import AdminAuthenticatedLayout from '@/Layouts/AdminAuthenticatedLayout.vue';
import { ref, onMounted, computed, reactive } from 'vue';
import { Head, usePage, useForm } from '@inertiajs/vue3';
import NavLink from '@/Components/NavLink.vue';

const page = usePage();
console.log(page.props);
const products = computed(() => page.props.products.data || []);
const carMakes = computed(() => page.props.carMakes || []);
const carModels = computed(() => page.props.carModels || []);
const carBodyStyles = computed(() => page.props.carBodyStyles || []);
const loading = computed(() => page.props.loading || false);
const perPageOptions = [10, 20, 50, 100]; // Adjust as needed
const perPage = ref(perPageOptions[0]);
const checkAll = ref(false);

const multiAction = ref('');
const firstpage = ref(null);
const lastpage = ref(null);
const nextpage = ref(null);
const previouspage = ref(null);
const totalPages = ref(null);
const currentPage = ref(null);
const urlPerPage = ref(page.props.per_page);
const flashMessage = ref('');
const isMobile = ref(window.innerWidth <= 768);

if(perPage != urlPerPage) {
    perPage.value = urlPerPage.value;
}

const productsChecked = reactive({});
const form = useForm({});

const sortDirections = ref({});
const sortBy = (key) => {
  // If the key has not been sorted before, default to ascending
  if (!sortDirections.value[key]) {
    sortDirections.value[key] = 'asc';
  } else {
    // If it was ascending before, now make it descending, and vice versa
    sortDirections.value[key] = sortDirections.value[key] === 'asc' ? 'desc' : 'asc';
  }

  const direction = sortDirections.value[key];

  products.value.sort((a, b) => {
    if (a[key] < b[key]) {
      return direction === 'asc' ? -1 : 1;
    }
    if (a[key] > b[key]) {
      return direction === 'asc' ? 1 : -1;
    }
    return 0;
  });
};

const handleResize = () => {
    isMobile.value = window.innerWidth <= 768; // Adjust the threshold as needed
}

const sortedProducts = computed(() => products.value);

const changePerPage = () => {
    window.location.href = getLinkUrl();
};

const getLinkUrl = () => {
  // Build the URL with the selected per page value
  return `/admin/products?per_page=${perPage.value}`;
};

const editProduct = (product) => {
  // Implement your edit logic here
  console.log('Edit product:', product);
  window.location.href = `/admin/product/edit/${product.id}`;
};

const deleteProduct = (product) => {
  const productID = product.id

  if (confirm('Are you sure you want to delete this product?')) {
    form.post(route('admin.products.delete',{id: productID}), {
        onSuccess: () => {
            // Update your success message or perform other actions here
            flashMessage.value = "Product deleted successfully!";
        },
        onError: (error) => {
            // Handle error case here, if needed
            console.error(error);
        }
    });
  }
};

const selectedAllItems = () => {
    console.log("selected all");
    if(checkAll.value) {
        checkAll.value = false;
    }
    else {
        checkAll.value = true;
    }

    products.value.forEach(product => {
        productsChecked[product.id] = checkAll.value;
    });
}

const removeChecked = (productID) => {
    console.log("Remove selected");
    productsChecked[productID] = false;
    checkAll.value = false;
}

const multiSelectProcess = () => {
    console.log("admin.products.multiselect");

};

const getCarMake = (productCarMake) => {
    let productCarMakeArray;
    if (Array.isArray(productCarMake)) {
        productCarMakeArray = productCarMake;
    } else if (typeof productCarMake === 'string') {
        productCarMakeArray = productCarMake.split(',').map(value => parseInt(value.trim(), 10));
    } else {
        productCarMakeArray = []; // Default empty array
    }

    const carMakeArray = [];

    productCarMakeArray.forEach(makeId => {
        const carMake = carMakes.value.find(item => item.id === makeId);
        if (carMake) {
            carMakeArray.push(carMake.name);
        }
    });

    return carMakeArray.join(', ');
}

const getCarModel = (productCarModel) => {

    let productCarModelArray;
    if (Array.isArray(productCarModel)) {
        productCarModelArray = productCarModel;
    } else if (typeof productCarModel === 'string') {
        productCarModelArray = productCarModel.split(',').map(value => parseInt(value.trim(), 10));
    } else {
        productCarModelArray = []; // Default empty array
    }

    const carModelArray = [];

    productCarModelArray.forEach(modelId => {
        const carModel = carModels.value.find(item => item.id === modelId);
        if (carModel) {
            carModelArray.push(carModel.name);
        }
    });

    return carModelArray.join(', ');
}

const getCarBodyStyle = (productCarBodyStyle) => {

    let productCarBodyStyleArray;
    if (Array.isArray(productCarBodyStyle)) {
        productCarBodyStyleArray = productCarBodyStyle;
    } else if (typeof productCarBodyStyle === 'string') {
        productCarBodyStyleArray = productCarBodyStyle.split(',').map(value => parseInt(value.trim(), 10));
    } else {
        productCarBodyStyleArray = []; // Default empty array
    }

    const carBodyStyleArray = [];

    productCarBodyStyleArray.forEach(bodyStyleId => {
        const carBodyStyle = carBodyStyles.value.find(item => item.id === bodyStyleId);
        if (carBodyStyle) {
            carBodyStyleArray.push(carBodyStyle.name);
        }
    });

    return carBodyStyleArray.join(', ');
}

const submit = () => {
    console.log("admin.products.multiselect");
    console.log(productsChecked);
    console.log(multiAction);
    form.post(route('admin.products.multiselect', {
        products: productsChecked,
        action: multiAction.value,
    }), {
        onSuccess: () => {
            console.log(multiAction);
            if(multiAction.value == 'instock') {
                flashMessage.value = "Selected Products marked as In-Stock";
            }
            else if(multiAction.value == 'delete') {
                flashMessage.value = "Selected Products deleted";
            }
            // setTimeout(() => {
            //     window.location.reload();
            // }, 2000);
        },
        onError: (error) => {
            console.error(error);
        }
    });
};

onMounted(() => {
    const query = usePage().props;
    firstpage.value = query.products.first_page_url;
    lastpage.value = query.products.last_page_url;
    nextpage.value = query.products.next_page_url;
    previouspage.value = query.products.prev_page_url;
    totalPages.value = query.products.last_page;
    currentPage.value = query.products.current_page;
    window.addEventListener('resize', handleResize);
    handleResize();

    products.value.forEach(product => {
        productsChecked[product.id] = false; // Default quantity
    });
});
</script>

<template>
    <Head title="All Products" />

    <AdminAuthenticatedLayout>
        <template #header>
            <div class="flex">
                <h2 class="font-semibold text-xl text-gray-800 leading-tight">Products
                    <NavLink :href="route('admin.products.add')" :active="route().current('admin.products.add')">
                        + Add Product
                    </NavLink>
                </h2>
            </div>
        </template>

        <div class="py-12">
            <div v-if="flashMessage" class="max-w-7xl mx-auto sm:px-6 lg:px-8 font-bold text-green-600">
                <div class="form-row mb-6">
                    <div class="col-md-6">
                        <h4>{{ flashMessage }}</h4>
                    </div>
                </div>
            </div>
            <form method="POST" @submit.prevent="submit">
            <div class="max-w-7xl mx-auto sm:px-6 lg:px-8">
                <div class="flex" style="width:400px;display:block;float:left;">
                    <select name="selectedType" v-model="multiAction">
                        <option value="">Select Action</option>
                        <option value="instock">Marked in Stock</option>
                        <option value="delete">Delete Selected</option>
                    </select>
                    <input class="btn bg-white sm:mx-3" type="submit" name="" value="Apply"/>
                </div>
                <nav class="flex" style="width:450px;display:block;float:right;">
                    <label for="perPageSelect">Per Page </label>
                    <select id="perPageSelect" class="sm:mx-3 sm:mb-6" v-model="perPage" @change="changePerPage">
                        <option v-for="option in perPageOptions" :key="option" :value="option">
                            {{ option }}
                        </option>
                    </select>
                    <ul class="pagination" style="float:right;">
                        <li class="page-item">
                            <a class="page-link" :href="firstpage" v-if="firstpage && currentPage != totalPages"><span aria-hidden="true">&laquo;</span></a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="previouspage + '&per_page='+perPage" v-if="previouspage">Prev</a>
                        </li>
                        <li class="page-item" style="text-align: center;">
                            <a class="page-link"> {{ currentPage }} of {{ totalPages }}</a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="nextpage + '&per_page='+perPage" v-if="nextpage">Next</a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="lastpage" v-if="lastpage && currentPage != totalPages"><span aria-hidden="true">&raquo;</span></a>
                        </li>
                    </ul>
                </nav>
                <div v-if="loading">Loading...</div>
                <table v-else-if="products && products.length" class="table table-striped">
                    <thead>
                        <tr>
                            <th><input type="checkbox" name="itemsCheckAll" id="itemsCheckAll" v-model="checkAll" @input="selectedAllItems" /></th>
                            <th @click="sortBy('item_number')">Item Number</th>
                            <th @click="sortBy('price')">Price</th>
                            <th v-if="!isMobile" @click="sortBy('year')">Year</th>
                            <th v-if="!isMobile && carMakes.length > 0" @click="sortBy('make')">Make</th>
                            <th v-if="!isMobile && carModels.length > 0" @click="sortBy('model')">Model</th>
                            <th v-if="!isMobile && carBodyStyles.length > 0" @click="sortBy('bodystyle')">Body Style</th>
                            <th @click="sortBy('quantity')">Stock</th>
                            <th>Actions</th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr v-for="product in sortedProducts" :key="product.id">
                            <td><input type="checkbox" v-model.number="productsChecked[product.id]" @input="removeChecked(product.id)" /></td>
                            <td>{{ product.item_number }}</td>
                            <td>{{ product.price }}</td>
                            <td v-if="!isMobile" >{{ product.year }}</td>
                            <td v-if="!isMobile && carMakes.length > 0" >{{ getCarMake(product.make) }}</td>
                            <td v-if="!isMobile && carModels.length > 0" >{{ getCarModel(product.model) }}</td>
                            <td v-if="!isMobile && carBodyStyles.length > 0" >{{ getCarBodyStyle(product.bodystyle) }}</td>
                            <td :class="product.stock == 1 ? 'text-success' : 'text-danger'">{{ product.stock == 1 ? "In Stock" : "Please Call" }}</td>
                            <td>
                                <button class="px-3" @click="editProduct(product)"><i class="fa fa-pen text-green-600"></i></button>
                                <button class="mx-3" @click="deleteProduct(product)"><i class="fa fa-trash text-red-600"></i></button>
                            </td>
                        </tr>
                    </tbody>
                </table>
                <p v-else>No products found.</p>
            </div>
            </form>
        </div>
    </AdminAuthenticatedLayout>
</template>
